package org.glitched.engine.graphic;

import java.util.ArrayList;
import org.glitched.util.Vector2f;
import org.lwjgl.opengl.GL11;
import org.newdawn.slick.Color;

/**
 * Renderer class : Classe principale de rendu, gère les sprites
 *
 * @author Sparamoule
 */
public class Renderer {
    
    private ArrayList<Sprite> sprites = new ArrayList<>();
    
    private Vector2f viewPosition = new Vector2f(0,0);
    
    public Renderer() {
        
    }
    
    public void draw() {
        
        GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glEnable(GL11.GL_ALPHA_TEST);
        GL11.glAlphaFunc(GL11.GL_GEQUAL, 1f);
        GL11.glDepthFunc(GL11.GL_LEQUAL);
        
        for (Sprite s : sprites) {
            // On applique la position du sprite
            Vector2f position = new Vector2f(s.getPosition().x,s.getPosition().y);
            // Ici si le sprite dépend de la vue, sinon on laisse comme c'est
            if (s.getDepth() > 0.f) {
                position.x -= viewPosition.x;
                position.y -= viewPosition.y;

                position.x /= s.getDepth();
                position.y /= s.getDepth();
            }
            
            
            s.getImage().getTexture().bind();
            GL11.glTranslatef(0,0,-1+((float)s.getLayer()/1000f));
             
            new Color(s.getColor().r, s.getColor().g, s.getColor().b,s.getOpacity()*s.getColor().a).bind();
             
            GL11.glBegin(GL11.GL_QUADS);
            s.getImage().drawEmbedded(position.x,position.y,s.getSize().x,s.getSize().y);
            GL11.glEnd();
            
            GL11.glTranslatef(0,0,1-((float)s.getLayer()/1000f));
            
        }
        
        GL11.glDisable(GL11.GL_ALPHA_TEST);
        GL11.glDisable(GL11.GL_DEPTH_TEST);
        
    }
    
    
    public void addSprite(Sprite s) {
        this.sprites.add(s);
    }
    
    public void removeSprite(Sprite s) {
        this.sprites.remove(s);
    }

    public Vector2f getViewPosition() {
        return viewPosition;
    }

    public void setViewPosition(Vector2f viewPosition) {
        this.viewPosition = viewPosition;
    }
    
    
    
}
