package org.glitched.engine.resource;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;

/**
 * ResourceManager class : Classe qui se charge de contenir toutes les ressources
 * du jeu.
 * 
 * Dans chaque catégorie, une ressource est associée à un string.
 *
 * @author Sparamoule
 */
public class ResourceManager {
    
    /**
     * Map des Images du jeu
     */
    private Map<String,Image> images = new HashMap<>();
    
    /**
     * Charge une image
     * @param name Nom qui servira ensuite
     * @param ref Chemin vers l'image
     * @return 
     */
    public boolean loadImage(String name, String ref){
        try {
            this.images.put(name, new Image(ref));
            return true;
        } catch (SlickException ex) {
            Logger.getLogger(ResourceManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return false;
    }
    
    /**
     * Décharge une image
     * @param name Nom de l'image
     * @return true si l'image existait
     */
    public boolean unloadImage(String name) {
        if (this.images.containsKey(name)) {
           this.images.remove(name);
           return true;
        } else {
            return false;
        }
    }
    
    /**
     * Retourne l'image qui est associée au nom
     * @param name Nom de l'image
     * @return l'objet Image associé au nom, null s'il n'existe pas.
     */
    public Image getImage(String name) {
        try {
            if (this.images.containsKey(name)) {
                return this.images.get(name);
            } else {
                throw new RuntimeException(name + " does not exist");
            }
        } catch (RuntimeException ex) {
            Logger.getLogger(ResourceManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

}
