package org.glitched.game.level;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Random;
import org.glitched.game.entity.*;
import org.glitched.game.event.Event;
import org.glitched.util.Vector2f;

/**
 * Level class :
 *
 * @author Sparamoule
 */
public class Level {
    
    /**
     * Temps écoulé du niveau en microsecondes
     */
    private long levelTime;
    
    private Vector2f viewPosition = new Vector2f();
    private boolean mouseCursor = false;
    
    private ArrayList<Entity> entities = new ArrayList<>();
    private Queue<Event> events = new LinkedList<>();
    
    private int collisionCount = 0;
    
    private Random random = new Random();
    
    public void update(float delta) {
        
        /*
         * Détection des collisions
         */
        collisionCount = 0;
        for (int i=0;i<entities.size()-1;i++) {
            Entity entityi = entities.get(i);
            if ( entityi instanceof IEntityCollidable) {
                for (int j = i+1; j<entities.size();j++) {
                   Entity entityj = entities.get(j);
                   if (entityj instanceof IEntityCollidable) {
                       if (((IEntityCollidable)entityi).getBounds().collide(entityi.getPosition().x, entityi.getPosition().y, ((IEntityCollidable)entityj).getBounds(), entityj.getPosition().x, entityj.getPosition().y)) {
                           ((IEntityCollidable)entityi).onCollide((IEntityCollidable)entityj);
                           ((IEntityCollidable)entityj).onCollide((IEntityCollidable)entityi);
                           // Attribution des dommages
                           if (entityi instanceof IEntityDamaging && entityj instanceof IEntityDamagable) {
                               applyDamage((IEntityDamaging)entityi,(IEntityDamagable)entityj);
                           } else if (entityj instanceof IEntityDamaging && entityi instanceof IEntityDamagable) {
                               applyDamage((IEntityDamaging)entityj,(IEntityDamagable)entityi);
                           }
                       }
                   }
                }
            }
            
        }
        
        this.levelTime += (int)(delta*1000000);
    }
    
    public void addEntity(Entity e) {
        this.entities.add(e);
    }
    
    public void removeEntity(Entity e) {
        this.entities.remove(e);
    }
    
    public ArrayList<Entity> getEntities() {
        return entities;
    }
    
    /**
     * Applique les dommages
     * @param entityDamaging Entité qui inflinge les dommages
     * @param entityDamagable Entité qui subit les dommages
     */
    private void applyDamage(IEntityDamaging entityDamaging, IEntityDamagable entityDamagable) {
        // Applique un dommage random
        float mindamage = entityDamaging.getMinDamage();
        float maxdamage = entityDamaging.getMaxDamage();
        float damage = random.nextFloat()*(maxdamage-mindamage) + mindamage;
        entityDamagable.addHealth(-damage);
        // Fais mourir l'entité
        if (entityDamagable.getHealth() <= 0.f) {
           entityDamagable.onDeath();
           entities.remove((Entity)entityDamagable);
        }
    }
    
    
    public void printLog() {
        
        System.out.println("levelTime=" + levelTime);
        System.out.println("viewPosition=" + viewPosition.x + ":" + viewPosition.y);
        System.out.println("mouseCursor=" + mouseCursor);
        System.out.println("collisionCount=" + collisionCount);
        for (int i=0;i<entities.size();i++) {
            Entity e = entities.get(i);
            System.out.println("Entity " + i + " : ");
            System.out.println(e.getPosition().x);
            System.out.println(e.getPosition().y);
        }
           
    }
    

}
