package org.glitched.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * FileLoader class : Ouvre un fichier et propose des extractions différentes
 *
 * @author Sparamoule
 */
public class FileLoader {
    
    /**
     * Chemin du fichier à ouvrir
     */
    private String path;
    
    /**
     * Contenu du fichier (après appel de load() )
     */
    private byte[] content;
    
    /**
     * Modifie le chemin du fichier
     * @param path 
     */
    public void setPath(String path) {
        this.path = path;
    }
    
    /**
     * Constructeur
     * @param path chemin du fichier à charger
     */
    public FileLoader(String path) {
        setPath(path);
    }
    
    public FileLoader() {}
    
    /**
     * Charge le contenu brut du fichier
     */
    public void load() {
        if (path == null) {
            throw new RuntimeException("Can't load file : path has not been specified");
        }
        
        File file = new File(path);
        FileInputStream fis;
        FileChannel fc;
        
        ByteBuffer bbuff = null;
        
        try {
            fis = new FileInputStream(file);
            fc = fis.getChannel();
            bbuff = ByteBuffer.allocateDirect((int)fc.size());
            fc.read(bbuff);
            
        } catch (IOException ex) {
            Logger.getLogger(FileLoader.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        bbuff.flip();
        content = bbuff.array();    
    }
    
    /**
     * Retourne le contenu brut du fichier
     * @return le tableau de bytes constituant le fichier
     */
    public byte[] getRawContent() {
        return content;
    }
    
    /**
     * Retourne un string contenant le fichier entier
     * @return 
     */
    public String getStringContent() {
        return new String(content);
    }
    
    
    /**
     * Retourne les lignes du fichier
     * @return un tableau de strings
     */
    public String[] getLineContent() {
        String c = getStringContent();
        if (c.contains("\r\n")) {
            return c.split("\r\n");
        } else if (c.contains("\n")) {
            return c.split("\n");
        } else if (c.contains("\r")) {
            return c.split("\r");
        } else {
            return new String[]{c};
        }
    }
    

}
